% % The MATLAB code of training algorithm and testing results for the NARX DCFS
% This code has been taken and adapted for NARX DCFS from the reference:
%
% Wang, L.-X. "Fast Training Algorithms for Deep Convolutional Fuzzy Systems with Application to Stock Index Prediction". 
% IEEE Trans. Fuzzy Syst. 2019, 1–1, doi:10.1109/TFUZZ.2019.2930488. 
%
% The MATLAB code of training algorithm and testing results for the NARX
% DCFS model for examples presented in chapter 4.2. Nonlinear Dynamic Test Processes Identification 

clear all
Proces='HAM'; %% TIP may be string: HAM, WIN, NDE, NSD
u = Generate_Inp_Sig('nprbs')*1.0; % May be 'tests' or 'lprbs'
yy = Second_order_DNS(u,Proces);
nu=3; ny=3; d=0;
data_matrix = In_Out_Str_DCFS(u,yy,nu,ny,d); % Function generates matrix for DCFS 
% data_matrix is N×7 matrix containing all the data, the first
% 6 columns are inputs and the last column is output.
[N,~]=size(data_matrix);
ntrain=round(N*2/3); % The first 2/3 are training data, and the
% remaining 1/3 are testing data.
Num_Fuzzy_Sets=20; % Num_Fuzzy_Sets is the number of fuzzy sets covering the input variables.
% NARX DCFS with nu=3, ny=3, n1=4, m^1=m^2=3,m^3=2 and L =3 is used.
for i=1:ntrain
  x11(i,1)=data_matrix(i,1);
  x11(i,2)=data_matrix(i,nu+1);
  x11(i,3)=data_matrix(i,2);  
  x12(i,1)=data_matrix(i,2);
  x12(i,2)=data_matrix(i,nu+2);
  x12(i,3)=data_matrix(i,3); 
  x13(i,1)=data_matrix(i,nu+1);
  x13(i,2)=data_matrix(i,2);
  x13(i,3)=data_matrix(i,nu+2);  
  x14(i,1)=data_matrix(i,nu+2);
  x14(i,2)=data_matrix(i,3);
  x14(i,3)=data_matrix(i,nu+3); 

  y(i,1)=data_matrix(i,7); % y is the desired output.
end;

% The following 4 lines train the 4 FS's in Level 1, where
% train_wm_deep trains the fuzzy system.
[y_center_11 MinMaxRange_11]=train_wm_deep(Num_Fuzzy_Sets,x11,y);
[y_center_12 MinMaxRange_12]=train_wm_deep(Num_Fuzzy_Sets,x12,y);
[y_center_13 MinMaxRange_13]=train_wm_deep(Num_Fuzzy_Sets,x13,y);
[y_center_14 MinMaxRange_14]=train_wm_deep(Num_Fuzzy_Sets,x14,y);
% The following 6 lines computes x21,…,x22 which are inputs to the 2 fuzzy
% systems in Level 2, where eval_wm_deep computes the output of the
% fuzzy system with parameters y_center and MinMaxRange.
x21(1:ntrain,1)=eval_wm_deep(Num_Fuzzy_Sets,y_center_11,MinMaxRange_11,x11);
x21(1:ntrain,2)=eval_wm_deep(Num_Fuzzy_Sets,y_center_12,MinMaxRange_12,x12);
x21(1:ntrain,3)=eval_wm_deep(Num_Fuzzy_Sets,y_center_13,MinMaxRange_13,x13);
x22(1:ntrain,1)=x21(:,2);
x22(1:ntrain,2)=x21(:,3);
x22(1:ntrain,3)=eval_wm_deep(Num_Fuzzy_Sets,y_center_14,MinMaxRange_14,x14);
% The following 2 lines train the 2 fuzzy systems in Level 2.
[y_center_21 MinMaxRange_21]=train_wm_deep(Num_Fuzzy_Sets,x21,y);
[y_center_22 MinMaxRange_22]=train_wm_deep(Num_Fuzzy_Sets,x22,y);
% % x31 are inputs to the fuzzy system in Level 3.
x31(1:ntrain,1)=eval_wm_deep(Num_Fuzzy_Sets,y_center_21,MinMaxRange_21,x21);
x31(1:ntrain,2)=eval_wm_deep(Num_Fuzzy_Sets,y_center_22,MinMaxRange_22,x22);
% Train the fuzzy system in Level 3.
[y_center_31 MinMaxRange_31]=train_wm_deep(Num_Fuzzy_Sets,x31,y);

% Computing the training and testing errors 
% The variables are the same.
for i=1:N
  x11(i,1)=data_matrix(i,1);
  x11(i,2)=data_matrix(i,nu+1);
  x11(i,3)=data_matrix(i,2);  
  x12(i,1)=data_matrix(i,2);
  x12(i,2)=data_matrix(i,nu+2);
  x12(i,3)=data_matrix(i,3); 
  x13(i,1)=data_matrix(i,nu+1);
  x13(i,2)=data_matrix(i,2);
  x13(i,3)=data_matrix(i,nu+2);  
  x14(i,1)=data_matrix(i,nu+2);
  x14(i,2)=data_matrix(i,3);
  x14(i,3)=data_matrix(i,nu+3); 
  
  y(i,1)=data_matrix(i,7); % y is the desired output.
end;

x21(1:N,1)=eval_wm_deep(Num_Fuzzy_Sets,y_center_11,MinMaxRange_11,x11);
x21(1:N,2)=eval_wm_deep(Num_Fuzzy_Sets,y_center_12,MinMaxRange_12,x12);
x21(1:N,3)=eval_wm_deep(Num_Fuzzy_Sets,y_center_13,MinMaxRange_13,x13);
x22(1:N,1)=x21(:,2);
x22(1:N,2)=x21(:,3);
x22(1:N,3)=eval_wm_deep(Num_Fuzzy_Sets,y_center_14,MinMaxRange_14,x14);
% Level 1 computing done
x31(1:N,1)=eval_wm_deep(Num_Fuzzy_Sets,y_center_21,MinMaxRange_21,x21);
x31(1:N,2)=eval_wm_deep(Num_Fuzzy_Sets,y_center_22,MinMaxRange_22,x22);
% Level 2 computing done
y_model=eval_wm_deep(Num_Fuzzy_Sets,y_center_31,MinMaxRange_31,x31); % yy is output of DCFS.
% 'Level 3 computing done'
y_model=y_model';

% The following computes and plots the training and testing errors.
for i=1:N
 e(i)=y(i)-y_model(i);
end;
err_train=0;
for i=1:ntrain
 err_train=err_train+e(i)^2;
end;
err_train=sqrt(err_train/ntrain);
err_test=0;
for i=ntrain+1:N
 err_test=err_test+e(i)^2;
end;
err_test=sqrt(err_test/(N-ntrain));

fig1=figure(1);
fig1.Position=[10 10 1120 840*4/4];

ax1=subplot(3,1,1), p1=plot(u(:,1),'b'); ax1.FontSize = 11;
axis([0 N min(u)-abs((max(u)-min(u)))*0.2 max(u)+abs((max(u)-min(u)))*0.2]), grid on, grid minor,p1.LineWidth=2;
ylabel('\itu\rm','FontSize',12), title('\rmInput signal','FontSize',12), 
line([ntrain ntrain],[min(u)-abs((max(u)-min(u)))*0.2 max(u)+abs((max(u)-min(u)))*0.2],'Color','Green','LineStyle','--','LineWidth',2)
legend(p1,'\itu\rm','FontSize',12,'TextColor','black')
ax2=subplot(3,1,2), p2=plot([y y_model]); ax2.FontSize = 11;
title('\rmOutput signal','FontSize',12), ylabel('\ity, y_m','FontSize',12), axis([0 N min(y)-abs((max(y)-min(y)))*0.2 max(y)+abs((max(y)-min(y)))*0.2]), grid on, grid minor, p2(1).LineWidth=1.5; p2(2).LineWidth=2;
line([ntrain ntrain],[min(y)-abs((max(y)-min(y)))*0.2 max(y)+abs((max(y)-min(y)))*0.2],'Color','Green','LineStyle','--','LineWidth',2)
legend(p2,'\ity\rm','\ity_m','FontSize',12,'TextColor','black','Location','best','NumColumns',2)
ax3=subplot(3,1,3), p3=plot(e,'r'); ax3.FontSize = 11;
title(['\rm\it\tau\rm = ' num2str(d) ',     \rm\itM\rm = ' num2str(Num_Fuzzy_Sets) ',     \rm\itMSE_{train}\rm = ' num2str(round(err_train,3)) ',     \rm\itMSE_{test}\rm = ' num2str(round(err_test,3))],'FontSize',12) 
xlabel('\itk \rm','FontSize',12), ylabel('\ite','FontSize',12), axis([0 N min(e)-(max(e)-min(e))*0.2 max(e)+(max(e)-min(e))*0.2]), grid on, grid minor, p3.LineWidth=2;
line([ntrain ntrain],[min(e)-abs((max(e)-min(e)))*0.2 max(e)+abs((max(e)-min(e)))*0.2],'Color','Green','LineStyle','--','LineWidth',2)
legend(p3,'\ite\rm','FontSize',12,'TextColor','black','Location','best')
text(ntrain-500,max(e)+(max(e)-min(e))*0.1,['Train'],'FontSize',12), text(ntrain+100,max(e)+(max(e)-min(e))*0.1,['Test'],'FontSize',12)

% za risanje
%save([Proces '_train.mat'])

% Testing with test data
u = Generate_Inp_Sig('tests');
yy = Second_order_DNS(u,Proces);
data_matrix = In_Out_Str_DCFS(u,yy,nu,ny,d);

% The meanings of the variables are the same as in training.
for i=1:N
  x11(i,1)=data_matrix(i,1);
  x11(i,2)=data_matrix(i,nu+1);
  x11(i,3)=data_matrix(i,2);  
  x12(i,1)=data_matrix(i,2);
  x12(i,2)=data_matrix(i,nu+2);
  x12(i,3)=data_matrix(i,3); 
  x13(i,1)=data_matrix(i,nu+1);
  x13(i,2)=data_matrix(i,2);
  x13(i,3)=data_matrix(i,nu+2);  
  x14(i,1)=data_matrix(i,nu+2);
  x14(i,2)=data_matrix(i,3);
  x14(i,3)=data_matrix(i,nu+3); 
  
  y(i,1)=data_matrix(i,7); % y is the desired output.
end;

x21(1:N,1)=eval_wm_deep(Num_Fuzzy_Sets,y_center_11,MinMaxRange_11,x11);
x21(1:N,2)=eval_wm_deep(Num_Fuzzy_Sets,y_center_12,MinMaxRange_12,x12);
x21(1:N,3)=eval_wm_deep(Num_Fuzzy_Sets,y_center_13,MinMaxRange_13,x13);
x22(1:N,1)=x21(:,2);
x22(1:N,2)=x21(:,3);
x22(1:N,3)=eval_wm_deep(Num_Fuzzy_Sets,y_center_14,MinMaxRange_14,x14);

x31(1:N,1)=eval_wm_deep(Num_Fuzzy_Sets,y_center_21,MinMaxRange_21,x21);
x31(1:N,2)=eval_wm_deep(Num_Fuzzy_Sets,y_center_22,MinMaxRange_22,x22);

y_model=eval_wm_deep(Num_Fuzzy_Sets,y_center_31,MinMaxRange_31,x31); % yy is output of DCFS.
y_model=y_model';

% The following computes and plots the training and testing errors.
for i=1:N
 e(i)=y(i)-y_model(i);
end;
err_test=0;
for i=ntrain+1:N
 err_test=err_test+e(i)^2;
end;
err_test=sqrt(err_test/(N-ntrain));

fig2=figure(2);
fig2.Position=[10 10 1120 840*4/4];

ax1=subplot(3,1,1), p1=plot(u(:,1),'b'); ax1.FontSize = 11; 
axis([0 N min(u)-abs((max(u)-min(u)))*0.05 max(u)+abs((max(u)-min(u)))*0.05]), grid on, grid minor,p1.LineWidth=2;
ylabel('\itu\rm','FontSize',12), title('\rmInput signal','FontSize',12), 
legend(p1,'\itu\rm','FontSize',12,'TextColor','black')
ax2=subplot(3,1,2), p2=plot([y y_model]); ax2.FontSize = 11;
title('\rmOutput signal','FontSize',12), ylabel('\ity, y_m','FontSize',12), axis([0 N min(y)-abs((max(y)-min(y)))*0.05 max(y)+abs((max(y)-min(y)))*0.05]), grid on, grid minor, p2(1).LineWidth=1.5; p2(2).LineWidth=2;
legend(p2,'\ity\rm','\ity_m','FontSize',12,'TextColor','black','Location','best','NumColumns',2)
ax3=subplot(3,1,3), p3=plot(e,'r'); ax3.FontSize = 11;
title(['\rm\it\tau\rm = ' num2str(d) ',     \rm\itM\rm = ' num2str(Num_Fuzzy_Sets) ',     \rm\itMSE_{test}\rm = ' num2str(round(err_test,3))],'FontSize',12) 
xlabel('\itk \rm','FontSize',12), ylabel('\ite','FontSize',12), axis([0 N min(e)-(max(e)-min(e))*0.05 max(e)+(max(e)-min(e))*0.05]), grid on, grid minor, p3.LineWidth=2;
legend(p3,'\ite\rm','FontSize',10,'TextColor','black','Location','best')
% save([Proces '_test.mat'])


function [u] = Generate_Inp_Sig(metoda)
% Generate PRBS or N_PRBS input signal
% 'lprbs' or 'nprbs' or 'tests'

% Tkon=30;
% Ts=0.01;
N_hold=30; % min hold time of signal.
Perioda=2000; Kanalov=1; N_period=4; Amin=-1; Amax=1;
u_prbs=idinput([Perioda, Kanalov, N_period],'prbs',[0 1/N_hold],[Amin,Amax]);
N=size(u_prbs,1);
if metoda == 'nprbs'
  n_prbs=[];
  n_prbs(1,:)=u_prbs(1)*rand;
  for i=2:N
    if u_prbs(i) ~= u_prbs(i-1)  
      n_prbs(i,:)=u_prbs(i)*rand;
    else
     n_prbs(i,:)=n_prbs(i-1,:);  
    end  
  end  
  u=n_prbs';
elseif metoda == 'lprbs'
  u=u_prbs';
elseif metoda == 'tests'
  for i=1:N
    if i<=N/20
        u(i)=0;  
    elseif i>N/20 && i<=2*N/20
        u(i)=Amin;
    elseif i>2*N/20 && i<=3*N/20
        u(i)=Amin+(Amax-Amin)/8;
    elseif i>3*N/20 && i<=4*N/20
        u(i)=Amin+2*(Amax-Amin)/8;
    elseif i>4*N/20 && i<=5*N/20
        u(i)=Amin+3*(Amax-Amin)/8;   
    elseif i>5*N/20 && i<=6*N/20
        u(i)=Amin+4*(Amax-Amin)/8;
    elseif i>6*N/20 && i<=7*N/20
        u(i)=Amin+5*(Amax-Amin)/8; 
    elseif i>7*N/20 && i<=8*N/20
        u(i)=Amin+6*(Amax-Amin)/8;
    elseif i>8*N/20 && i<=9*N/20
        u(i)=Amin+7*(Amax-Amin)/8;   
    elseif i>9*N/20 && i<=10*N/20
        u(i)=Amin+8*(Amax-Amin)/8;
    elseif i>10*N/20 && i<=15*N/20
        u(i)=u(i-1)-(Amax-Amin)/(15*N/20 -10*N/20);  
    elseif i>15*N/20 && i<=20*N/20
        u(i)=1*sin((i-15*N/20-1)*2*pi/(20*N/20 - 15*N/20));        
    else
        disp('Error')
    end 
  end
else
  u=0;
  disp('Wrong choice of prbs or aprbs parameter')
end  
u=u'; 
end

function [y] = Second_order_DNS(u,TIP)
% The following lines generate output of the Second Order Dynamic Non-linear System. 
% Parameters are: u - vektor of input signal, TIP - vsrta nelinearnega
% sistema.
% TIP may be string: HAM, WIN, NDE, NSD
% 'HAM' Second Order Hammerstain System
% 'WIN' Second Order Winer System
% 'NDE' Nonlinear Diferential Equation
% 'NSD' Not separable dynamic and static system

% Second Order Hammerstain System plus noise
y(1)=0;
y(2)=0;
for k=3:size(u,1)
    if TIP=='HAM' 
        y(k) = 0.01867*atan(u(k-1)) + 0.01746*atan(u(k-2)) + 1.7826*y(k-1) - 0.8187*y(k-2);  
    elseif TIP=='WIN' 
        y(k) = atan(0.01867*u(k-1) + 0.01746*u(k-2) + 1.7826*tan(y(k-1)) - 0.8187*tan(y(k-2)));
    elseif TIP=='NDE' 
        y(k) = -0.07289*(u(k-1)-0.2*y(k-1)^2) + 0.09394*(u(k-2)-0.2*y(k-2)^2) + 1.68364*y(k-1)-0.70469*y(k-2); 
    elseif TIP=='NSD' 
        y(k) = 0.133*u(k-1)-0.0667*u(k-2)+1.5*y(k-1)-0.7*y(k-2)+u(k)*(0.1*y(k-1)-0.2*y(k-2));
    else
        disp('napačna izbira sistema') 
    end   
   % y(k) = y(k)+0.0001*randn;
end    
y=y';
end

function [data_matrix] = In_Out_Str_DCFS(u,y,nu,ny,d)
% The following lines generate data_matrix, which is Nx(nu+ny) matrix containing all the data, the first
% 5 columns are inputs u(k-2),y(k-2),u(k-1),y(k-1),u(k) and the last column is output y(k).
N=size(y,1);
l=max(nu+d,ny);
for i=1:N-l
 k=1;   
 for j=l-d:-1:l-d-nu+1
     data_matrix(i,k)=u(i+j-1);
     k=k+1;
 end
 k=1;
 for j=l:-1:l-ny+1
     data_matrix(i,nu+k)=y(i+j-1);
     k=k+1;
 end
 data_matrix(i,nu+ny+1)=y(i+l);
end 
end

function [y_center ranges]=train_wm_deep(Num_Fuzzy_Sets,data_matrix,y)
% Train fuzzy system with input data_matrix and output y to get y_center and ranges,
% where y_center is the in WM FS and ranges is the endpoints in.
% Original function is here:  https://ieeexplore.ieee.org/ielx7/91/9130783/8788632/code_training_algorithm.pdf?arnumber=8788632
%
extra=[data_matrix,y];
[numSamples,m]=size(extra);
numInput=m-1;
for i=1:numInput
  fnCounts(i)=Num_Fuzzy_Sets;
end;
ranges = zeros(numInput,2);
activFns = zeros(numInput,2);
activGrades = zeros(numInput,2);
searchPath = zeros(numInput,2);
numCells = 1; % number of regions (cells)
for i = 1:numInput
  ranges(i,1) = min(extra(:,i));
  ranges(i,2) = max(extra(:,i));
  numCells = numCells * fnCounts(i);
end;
baseCount(1)=1;
for i=2:numInput
  baseCount(i)=1;
for j=2:i
  baseCount(i)=baseCount(i)*fnCounts(numInput-j+2);
end;
end;
% Generate rules for cells covered by data
y_center = zeros(1,numCells); % THEN part centers of generated rules
ym = zeros(1,numCells);
for k = 1:numSamples
  for i = 1:numInput
    numFns = fnCounts(i);
    nthActive = 1;
    for nthFn = 1:numFns
      grade = Membership_at_x(numFns,nthFn,extra(k,i),ranges(i,1),ranges(i,2));
      if grade > 0
        activFns(i,nthActive) = nthFn;
        activGrades(i,nthActive) = grade;
        nthActive = nthActive + 1;
      end;
    end; % endfor nthFn
  end; % endfor i
  for i=1:numInput
    if activGrades(i,1) >= activGrades(i,2)
      searchPath(i,1)=activFns(i,1);
      searchPath(i,2)=activGrades(i,1);
    else
      searchPath(i,1)=activFns(i,2);
      searchPath(i,2)=activGrades(i,2);
    end;
  end;
  indexcell=1;
  grade=1;
  for i=1:numInput
    grade=grade*searchPath(i,2);
    indexcell=indexcell+(searchPath(numInput-i+1,1)-1)*baseCount(i);
  end;
  ym(indexcell)=ym(indexcell)+grade;
  y_center(indexcell)=y_center(indexcell)+extra(k,numInput+1)*grade;
end; % endfor k
for j=1:numCells
  if ym(j) ~= 0
    y_center(j)=y_center(j)/ym(j);
  end;
end;
% Extrapolate the rules to all the cells
for i=1:numInput-1
  baseCount(i)=1;
  for j=1:i
    baseCount(i)=baseCount(i)*fnCounts(numInput-j+1);
  end;
end;
ct=1;
y_center_b = zeros(1,numCells);
yNum_Fuzzy_Sets = zeros(1,numCells);
while ct > 0
  ct=0;
  for s=1:numCells
    if ym(s) == 0
      ct=ct+1;
    end;
  end;
  for s=1:numCells
    if ym(s) == 0
      s1=s;
      index = ones(1,numInput);
      for i=numInput-1:-1:1
        while s1 > baseCount(i)
          s1=s1-baseCount(i);
          index(numInput-i)=index(numInput-i)+1;
        end;
      end;
      index(numInput)=s1;
      y_center_num=0;
      for i=1:numInput-1
        if index(i) > 1
          y_center_b(s)=y_center_b(s)+y_center(s-baseCount(numInput-i));
          yNum_Fuzzy_Sets(s)=yNum_Fuzzy_Sets(s)+ym(s-baseCount(numInput-i));
          y_center_num=y_center_num+sign(ym(s-baseCount(numInput-i)));
        end;
        if index(i) < fnCounts(i)
          y_center_b(s)=y_center_b(s)+y_center(s+baseCount(numInput-i));
          yNum_Fuzzy_Sets(s)=yNum_Fuzzy_Sets(s)+ym(s+baseCount(numInput-i));
          y_center_num=y_center_num+sign(ym(s+baseCount(numInput-i)));
        end;
      end;
      if index(numInput) > 1
        y_center_b(s)=y_center_b(s)+y_center(s-1);
        yNum_Fuzzy_Sets(s)=yNum_Fuzzy_Sets(s)+ym(s-1);
        y_center_num=y_center_num+sign(ym(s-1));
      end;
      if index(numInput) < fnCounts(numInput)
        y_center_b(s)=y_center_b(s)+y_center(s+1);
        yNum_Fuzzy_Sets(s)=yNum_Fuzzy_Sets(s)+ym(s+1);
        y_center_num=y_center_num+sign(ym(s+1));
      end;
      if y_center_num >= 1
        y_center_b(s)=y_center_b(s)/y_center_num;
        yNum_Fuzzy_Sets(s)=yNum_Fuzzy_Sets(s)/y_center_num;
      end;
    end; % endif ym
  end; % endfor s
  for s=1:numCells
    if ym(s) == 0 & yNum_Fuzzy_Sets(s) ~= 0
      y_center(s)=y_center_b(s);
      ym(s)=yNum_Fuzzy_Sets(s);
    end;
  end;
end; % endwhile ct
end

function yy=eval_wm_deep(Num_Fuzzy_Sets,y_center,ranges,data_matrix)
% Compute the output of WM fuzzy system with y_center and ranges for input data_matrix.
% Original function is here:  https://ieeexplore.ieee.org/ielx7/91/9130783/8788632/code_training_algorithm.pdf?arnumber=8788632

exapp=data_matrix;
[numSamples,m]=size(exapp);
numInput=m;
for i=1:numInput
  fnCounts(i)=Num_Fuzzy_Sets;
end;
activFns = zeros(numInput,2);
activGrades = zeros(numInput,2);
baseCount(1)=1;
for i=2:numInput
  baseCount(i)=1;
for j=2:i
baseCount(i)=baseCount(i)*fnCounts(numInput-j+2);
end;
end;
for j=1:numInput
 for i1=1:2^j
  for i2=1:2^(numInput-j)
   ma(i2+(i1-1)*2^(numInput-j),j)=mod(i1-1,2)+1;
  end;
 end;
end;
e1sum=0;
for k = 1:numSamples
 for i = 1:numInput
  numFns = fnCounts(i);
  nthActive = 1;
  for nthFn = 1:numFns
   grade = Membership_at_x(numFns,nthFn,exapp(k,i),ranges(i,1),ranges(i,2));
   if grade > 0
    activFns(i,nthActive) = nthFn;
    activGrades(i,nthActive) = grade;
    nthActive = nthActive + 1;
   end;
  end;
 end;
 for i=1:numInput
  nn(i,1)=activFns(i,1);
  nn(i,2)=nn(i,1)+1;
  if nn(i,1) == fnCounts(i)
   nn(i,2)=nn(i,1);
  end;
 end;
 a=0;
 b=0;
 for i=1:2^numInput
  indexcell=1;
  grade=1;
  for j=1:numInput
   grade=grade*activGrades(j,ma(i,j));
   indexcell=indexcell+(nn(numInput-j+1,ma(i,numInput-j+1))-1)*baseCount(j);
  end;
  a=a+y_center(indexcell)*grade;
  b=b+grade;
 end;
 yy(k)= a/b; % the fuzzy system output
end; % endfor k
end

function y=Membership_at_x(n,i,x,xmin,xmax)
% Compute the value of the i’th membership function in Fig. 2 at x.
% Original function is here:  https://ieeexplore.ieee.org/ielx7/91/9130783/8788632/code_training_algorithm.pdf?arnumber=8788632

h=(xmax-xmin)/(n-1);

if i==1
 if x < xmin
  y=1;
 end;
 if x >= xmin & x < xmin+h
  y=(xmin-x+h)/h;
 end;
 if x >= xmin+h
  y=0;
 end;
end;

if i > 1 & i < n
 if x < xmin+(i-2)*h | x > xmin+i*h
  y=0;
 end;
 if x >= xmin+(i-2)*h & x < xmin+(i-1)*h
  y=(x-xmin-(i-2)*h)/h;
 end;
 if x >= xmin+(i-1)*h & x <= xmin+i*h
  y=(-x+xmin+i*h)/h;
 end;
end;

if i==n
 if x < xmax-h
  y=0;
 end;
 if x >= xmax-h & x < xmax
  y=(-xmax+x+h)/h;
 end;
 if x >= xmax
  y=1;
 end;
end;
end